@@ -31,6 +31,35 @@ e_mdn_response_policy_get_type (void)
 	return the_type__volatile;
 }
 
+GType
+e_proxy_method_get_type (void)
+{
+	static volatile gsize the_type__volatile = 0;
+
+	if (g_once_init_enter (&the_type__volatile)) {
+		static const GEnumValue values[] = {
+			{ E_PROXY_METHOD_DEFAULT,
+			  "E_PROXY_METHOD_DEFAULT",
+			  "default" },
+			{ E_PROXY_METHOD_MANUAL,
+			  "E_PROXY_METHOD_MANUAL",
+			  "manual" },
+			{ E_PROXY_METHOD_AUTO,
+			  "E_PROXY_METHOD_AUTO",
+			  "auto" },
+			{ E_PROXY_METHOD_NONE,
+			  "E_PROXY_METHOD_NONE",
+			  "none" },
+			{ 0, NULL, NULL }
+		};
+		GType the_type = g_enum_register_static (
+			g_intern_static_string ("EProxyMethod"),
+			values);
+		g_once_init_leave (&the_type__volatile, the_type);
+	}
+	return the_type__volatile;
+}
+
 GType
 e_source_authentication_result_get_type (void)
 {
@@ -89,5 +118,6 @@ e_trust_prompt_response_get_type (void)
 	return the_type__volatile;
 }
 
+
 /* Generated data ends here */
 
